//+------------------------------------------------------------------+
//|                                                                  |
//| original NonLag moving average developed by igorad               |
//| this version made by mladen                                      | 
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1  DeepSkyBlue
#property indicator_width1  2

//
//
//
//
//

extern string TimeFrame   = "Current time frame";
extern int    Length      = 14;
extern int    Price       = PRICE_CLOSE;
extern bool   Interpolate = true;

//
//
//
//
//

double NonLagMaBuffer[];
double alphas[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{

   SetIndexBuffer(0,NonLagMaBuffer);
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
      calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
      timeFrame = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+" non lag MA ("+Length+")");
   return(0);  
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
         //
         //
         //    called to find out how many bars in the target
         //    time frame changed - just return the desired 
         //    number of bars that will be recalculated
         //
         //
         
               if (returnBars) { NonLagMaBuffer[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //    this is a calculation part done when time frame is same 
   //    as the target time frame or when indicator is called from
   //    another instance of itself in order to calculate values of
   //    some time frame. 
   //
   //    This part is skipped if the time frame is not the desired target time frame
   //
   //
   //
   
         if (calculateValue || timeFrame == Period())
         {
            for (i=limit; i>=0; i--) NonLagMaBuffer[i] = iNoLagMa(iMA(NULL,0,1,0,MODE_SMA,Price,i),Length,alphas,0,i);
            
            //
            //
            //    return(0) must stay here, otherwise it will crash mt :)
            //
            //
   	      
   	      return(0);	
         }	  



   //
   //
   //
   //    mtf part of calculation : indicator gets here only if "other" time frame (different than chart time frame) is asked for
   //
   //    first check hown many bars in the target time frame has changed (the "returnBars" in iCustom parameters)
   //    then do the actual multi time frame calculation by calling the indicator itself to calculate values (code above this comment)
   //
   //
   //
   

      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      for (i=limit;i>=0;i--)
      {
         int y = iBarShift(NULL,timeFrame,Time[i]);
            NonLagMaBuffer[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,Price,0,y);

            //
            //
            //
            //
            //
      
            if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
            if (!Interpolate) continue;

            //
            //
            //    do the liner interpolation if required by Interpolate parameter
            //
            //

            datetime time = iTime(NULL,timeFrame,y);
               for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(int k = 1; k < n; k++)
                  NonLagMaBuffer[i+k] = NonLagMaBuffer[i] + (NonLagMaBuffer[i+n] - NonLagMaBuffer[i])*k/n;
      }
   
   //
   //
   //
   //
   //
      
   return(0);
}


//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double values[][3];
double prices[][1];

#define Pi       3.14159265358979323846264338327950288
#define _length  0
#define _len     1
#define _weight  2

//
//
//
//
//

double iNoLagMa(double price, int length, double &alphasArray[], int forValue, int i)
{
   if (ArrayRange(prices,0) != Bars) ArrayResize(prices,Bars);
            int r = Bars-i-1;  prices[r][forValue]=price;
   if (length<3 || r<3) return(prices[r][forValue]);
   
   //
   //
   //
   //
   //
   
   if (ArrayRange(values,0)<(forValue+1) || values[forValue][_length] != length)
   {
      double Cycle = 4.0;
      double Coeff = 3.0*Pi;
      int    Phase = length-1;
      
      if (ArrayRange(values,0)<forValue+1) ArrayResize(values,forValue+1);

         values[forValue][_length] = length;
         values[forValue][_len]    = length*4 + Phase;  
         values[forValue][_weight] = 0;  
         ArrayResize(alphasArray,values[forValue][_len]);

         for (int k=0; k<values[forValue][_len]-1; k++)
         {
            if (k<=Phase-1) 
                 double t = 1.0 * k/(Phase-1);
            else        t = 1.0 + (k-Phase+1)*(2.0*Cycle-1.0)/(Cycle*length-1.0); 
            double beta = MathCos(Pi*t);
            double g = 1.0/(Coeff*t+1); if (t <= 0.5 ) g = 1;
      
            alphasArray[k]             = g * beta;
            values[forValue][_weight] += alphasArray[k];
         }
   }
   
   //
   //
   //
   //
   //
   
   if (values[forValue][_weight]>0)
   {
      int    len = values[forValue][_len];
      double sum = 0;
           for (k=0; k < len-1; k++) sum += alphasArray[k]*prices[r-k][forValue];      
           return( sum / values[forValue][_weight]);
   }
   else return(0);           
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}